# -*- coding: UTF-8 -*-
# Copyright 2020 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

from __future__ import unicode_literals
# from builtins import str

from atelier.utils import isidentifier

from lino.api import dd, _, pgettext


